package auth

import (
	"fmt"
	"net/http"
	"path"

	lfcc "git.loafle.net/commons_go/config"
	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_probes/central/api/module"
	"git.loafle.net/overflow/overflow_probes/central/client"
	"git.loafle.net/overflow/overflow_probes/commons"
	"git.loafle.net/overflow/overflow_probes/config"
	opuu "git.loafle.net/overflow/overflow_probes/util/url"
)

const (
	noAuthEntryPoint = "/auth"
)

type auth struct {
	*commons.Handlers
	c        client.Client
	entryURL string

	noAuthConfigPath string
	noAuthConfig     config.NoAuthProbeConfig

	acceptedChan chan bool
	deniedChan   chan error
}

func New() (AuthHandler, error) {
	var err error

	a := &auth{
		acceptedChan: make(chan bool),
		deniedChan:   make(chan error),
	}
	a.Handlers = commons.NewHandlers()

	if a.entryURL, err = opuu.Join(config.CFG.Central.URL, noAuthEntryPoint); nil != err {
		return nil, err
	}

	a.noAuthConfigPath = path.Join(*config.ConfigDir, config.NoAuthProbeConfigFileName)

	conf := lfcc.New()
	if lfcc.Exists(a.noAuthConfigPath) {
		if err = conf.Load(&a.noAuthConfig, a.noAuthConfigPath); nil != err {
			return nil, fmt.Errorf("Auth: Loading of NoAuth config file[%s] failed error[%v]", a.noAuthConfigPath, err)
		}
	}

	a.c = client.New()
	a.c.OnNotify(a.onNotify)

	return a, nil
}

func (a *auth) serveRegistration() error {
	var err error
	header := http.Header{}

	var enc string
	if enc, err = getRegistHeader(); nil != err {
		return err
	}
	header[module.NoAuthProbeHeader_NoAuthRegist] = []string{enc}

	var res *http.Response
	if res, err = a.c.Dial(a.entryURL, header); nil != err {
		return err
	}

	tempKey := res.Header.Get(module.NoAuthProbeHeader_SetNoAuthID)
	a.noAuthConfig.TempKey = &tempKey
	if err = lfcc.Save(a.noAuthConfig, a.noAuthConfigPath, true); nil != err {
		return err
	}

	return nil
}

func (a *auth) serveConnect(noAuthTempKey string) error {
	var err error
	header := http.Header{}
	header[module.NoAuthProbeHeader_NoAuthID] = []string{noAuthTempKey}

	var res *http.Response
	if res, err = a.c.Dial(a.entryURL, header); nil != err {
		return err
	}

	logging.Logger.Debug(fmt.Sprintf("Auth: Connect HTTP Status[%s]", res.Status))

	return nil
}
